/**
  *  \file client/tiles/objheader.cc
  *
  *  (c) 2001-2016 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/tiles/objheader.h"
#include "game/obj.h"
#include "gfx/palette.h"
#include "client/marks.h"
#include "ui/uiglobal.h"
#include "util/unicodechars.h"
#include "game/objsel.h"

/************************* WObjectSelectionHeaderTile *************************/

WObjectSelectionHeaderTile::WObjectSelectionHeaderTile(GObjectSelection& type)
    : WObjectSelectionObserverTile(font_sizes[FONT_NORMAL_BOLD].scaledBy(20, 1), type),
      button_next(0, UTF_DOWN_ARROW, SDLK_PAGEDOWN),
      button_prev(0, UTF_UP_ARROW,   SDLK_PAGEUP)
{
    init();
}

void
WObjectSelectionHeaderTile::drawData(GfxCanvas& can)
{
    string_t name;
    bool mark = false;
    if (GObject* o = getCurrentObject()) {
        name = o->getName(GObject::LongName);
        name += ":";
        mark = o->isMarked();
    }

    GfxContext ctx;
    ctx.useCanvas(can);
    ctx.useColorScheme(getSkin());
    ctx.useFont(*fonts[FONT_NORMAL_BOLD]);

    int w   = extent.w - 2*font_heights[FONT_NORMAL_BOLD];
    int ofs = 0;
    drawBackground(ctx, GfxRect(0, 0, w, extent.h));

    if (mark) {
        ctx.setColor(UIColor::tc_Selection);
        drawSelection(ctx, GfxPoint(5, 7), 1, 2);
        ofs = 15;
        w -= 15;
    }

    ctx.setColor(UIColor::tc_Static);
    outTextF(ctx, ofs, 0, w, name);
}

void
WObjectSelectionHeaderTile::getLayoutInfo(LayoutInfo& info)
{
    WObjectSelectionObserverTile::getLayoutInfo(info);
    info.flags |= li_FlexibleH;
}

void
WObjectSelectionHeaderTile::init()
{
    const int h = font_heights[FONT_NORMAL];
    setLayout(RightJustified, 0, false);
    button_prev.setState(UIButton::bf_SmallSize, true);
    button_prev.addCall(this, &WObjectSelectionHeaderTile::onPrev);
    button_next.setState(UIButton::bf_SmallSize, true);
    button_next.addCall(this, &WObjectSelectionHeaderTile::onNext);
    addControl(GfxRect(0, 0, h-1, h-1), button_next);
    addControl(GfxRect(h, 0, h-1, h-1), button_prev);
}

void
WObjectSelectionHeaderTile::onNext()
{
    if (int i = getObjectType()->findNextIndexWrap(getObjectSelection().getCurrentIndex(), false)) {
        getObjectSelection().setCurrentIndex(i);
    }
}

void
WObjectSelectionHeaderTile::onPrev()
{
    if (int i = getObjectType()->findPreviousIndexWrap(getObjectSelection().getCurrentIndex(), false)) {
        getObjectSelection().setCurrentIndex(i);
    }
}
