/**
  *  \file client/widgets/specbaseinfo.cc
  *  \brief Ship Specification Base Information
  *
  *  (c) 2001-2016 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/widgets/specbaseinfo.h"
#include "resmgr/resmgr.h"
#include "ui/table.h"
#include "game/spec.h"
#include "util/nls.h"
#include "util/misc.h"
#include "resmgr/resid.h"
#include "ui/uiglobal.h"

WSpecBaseInfo::WSpecBaseInfo()
    : image(105+2, 93+2),
      hull(0)
{
    image.setImage(requestPixmap(ResId("nvc")));
    add(image, GfxRect(getExtent().x, getExtent().y, 105+2, 93+2));
}

WSpecBaseInfo::~WSpecBaseInfo()
{ }

void
WSpecBaseInfo::getLayoutInfo(LayoutInfo& info)
{
    info.pref_size = info.min_size = GfxPoint(300, 6*font_heights[FONT_NORMAL]);
    info.flags = li_FlexibleH;
}

void
WSpecBaseInfo::drawContent(GfxCanvas& can)
{
    GfxRect r;
    r.x = image.getExtent().getRightX()+5;
    r.y = getExtent().y;
    r.w = getExtent().getRightX() - r.x;
    r.h = getExtent().h;

    GfxContext ctx;
    ctx.useCanvas(can).useColorScheme(getSkin());

    if (hull > 0) {
        UITableWriter tw(ctx, r, FONT_NORMAL);
        const GHull& h = ::getHull(hull);
        tw.newLColumn();
        tw.writeFixedCell(UIColor::tc_Static, _("Mass:"));
        tw.writeFixedCell(UIColor::tc_Static, _("Engines:"));
        tw.writeFixedCell(UIColor::tc_Static, _("Tech:"));
        tw.writeFixedCell(UIColor::tc_Static, _("Crew:"));
        tw.writeFixedCell(UIColor::tc_Static, _("Cargo:"));
        tw.writeFixedCell(UIColor::tc_Static, _("Fuel:"));

        tw.newRColumn(5*font_ems[FONT_NORMAL]);
        tw.writeVariableCell(UIColor::tc_Green, numToString(h.getMass()));
        tw.writeVariableCell(UIColor::tc_Green, numToString(h.getNumEngines()));
        tw.writeVariableCell(UIColor::tc_Green, numToString(h.getTech()));
        tw.writeVariableCell(UIColor::tc_Green, numToString(h.getCrew()));
        tw.writeVariableCell(UIColor::tc_Green, numToString(h.getCargo()));
        tw.writeVariableCell(UIColor::tc_Green, numToString(h.getFuel()));

        tw.newLColumn();
        tw.writeFixedCell(UIColor::tc_Green, _("kt"));
        tw.writeFixedCell(UIColor::tc_Green, "");
        tw.writeFixedCell(UIColor::tc_Green, "");
        tw.writeFixedCell(UIColor::tc_Green, "");
        tw.writeFixedCell(UIColor::tc_Green, _("kt"));
        tw.writeFixedCell(UIColor::tc_Green, _("kt"));

        tw.finish();
    } else {
        drawBackground(ctx, r);
    }
    can.update(r);
}

void
WSpecBaseInfo::setHull(int hull)
{
    if (hull != this->hull) {
        this->hull = hull;
        if (hull > 0) {
            image.setImage(requestPixmap(ResId(res::ship, ::getHull(hull).getInternalPictureNumber(), hull)));
        } else {
            image.setImage(requestPixmap(ResId("nvc")));
        }
        drawWidget();
    }
}
