/**
  *  \file gfx/clip.h
  *  \brief Simple Clipping
  *
  *  Classes for clipping output at a single rectangle.
  *
  *  (c) 2001-2016 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GFX_CLIP_H
#define PCC_V2_GFX_CLIP_H

#include "gfx/gfx.h"

class GfxClipRect : public GfxFilter {
 protected:
    GfxRect rect;
    GfxClipRect(const GfxRect& arect, GfxCanvas& parent);
 public:
    void drawHLine(const GfxPoint& pt, int npix, uint32 color, uint8 pat, alpha_type alpha);
    void drawVLine(const GfxPoint& pt, int npix, uint32 color, uint8 pat, alpha_type alpha);
    void drawPixel(const GfxPoint& pt, uint32 color, alpha_type alpha);
    void drawBar(GfxRect rect, uint32 color, uint32 bg, const GfxFillPattern& pat, alpha_type alpha);

    void blitSurface(const GfxPoint& pt, GfxCanvas& src, GfxRect rect);
    void blitPattern(GfxRect rect, const GfxPoint& pt, int bytes_per_line, const uint8* data, uint32 color, uint32 bg, alpha_type alpha);

    GfxRect computeClipRect(GfxRect r);
    bool computeOffset(GfxCanvas& other, GfxPoint& pt);
};

class GfxClipRectFilter : public GfxClipRect {
 public:
    GfxClipRectFilter(const GfxRect& arect, GfxCanvas& parent);
    void setClipRect(const GfxRect& arect) { rect = arect; }
};

class GfxOffset : public GfxFilter {
 protected:
    int dx, dy;
 public:
    GfxOffset(int adx, int ady, GfxCanvas& parent);

    void drawHLine(const GfxPoint& pt, int npix, uint32 color, uint8 pat, alpha_type alpha);
    void drawVLine(const GfxPoint& pt, int npix, uint32 color, uint8 pat, alpha_type alpha);
    void drawPixel(const GfxPoint& pt, uint32 color, alpha_type alpha);
    void drawBar(GfxRect rect, uint32 color, uint32 bg, const GfxFillPattern& pat, alpha_type alpha);

    void blitSurface(const GfxPoint& pt, GfxCanvas& src, GfxRect rect);
    void blitPattern(GfxRect rect, const GfxPoint& pt, int bytes_per_line, const uint8* data, uint32 color, uint32 bg, alpha_type alpha);

    GfxRect computeClipRect(GfxRect r);
    bool computeOffset(GfxCanvas& other, GfxPoint& pt);
    void update(GfxRect r);

    uint32 getPixel(const GfxPoint& pt);
};

#endif
