/**
  *  \file int/args.cc
  *  \brief Interpreter: Argument Handling Utilities
  *
  *  (c) 2001-2016 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "int/args.h"
#include "int/data.h"
#include "int/interror.h"

/****************************** IntArgBlock ******************************/

/** Get next argument. If there are no more arguments, returns 0,
    corresponding to an "empty" argument. */
IntValue*
IntArgBlock::getNext()
{
    if (nargs != 0) {
        --nargs;
        return dseg[index++];
    } else {
        return 0;
    }
}

/** Check that there are exactly \c need arguments. If not, throws an
    appropriate IntError. */
void
IntArgBlock::checkArgumentCount(uint32_t need)
{
    ::checkArgumentCount(nargs, need, need);
}

/** Check that there are \c min to \c max arguments. If not, throws an
    appropriate IntError. */
void
IntArgBlock::checkArgumentCount(uint32_t min, uint32_t max)
{
    ::checkArgumentCount(nargs, min, max);
}

/** Check that there are at least \c min arguments. If not, throws an
    appropriate IntError. */
void
IntArgBlock::checkArgumentCountAtLeast(uint32_t min)
{
    ::checkArgumentCount(nargs, min, nargs);
}

/******************************** Globals ********************************/

/** Check argument count.
    \param have Number of arguments we have
    \param min  Minimum number required
    \param max  Maximum number accepted
    \throw IntError if constraint violated */
void
checkArgumentCount(uint32_t have, uint32_t min, uint32_t max)
{
    if (have < min)
        throw IntError("Too few arguments");
    if (have > max)
        throw IntError("Too many arguments");
}
